@@ -6,9 +6,10 @@ import six
 from dagster import check
 from dagster.config.field_utils import check_user_facing_opt_config_param
 from dagster.core.definitions.config import ConfigMapping
+from dagster.core.definitions.config_mappable import IConfigMappable
 from dagster.core.errors import DagsterInvalidDefinitionError
 from dagster.utils import frozendict, frozenlist
-from dagster.utils.backcompat import canonicalize_backcompat_args, rename_warning
+from dagster.utils.backcompat import rename_warning
 
 from .dependency import SolidHandle
 from .input import InputDefinition, InputMapping
@@ -106,11 +107,11 @@ class ISolidDefinition(six.with_metaclass(ABCMeta)):
 
     @property
     def has_configurable_inputs(self):
-        return any([inp.dagster_type.input_hydration_config for inp in self._input_defs])
+        return any([inp.dagster_type.loader for inp in self._input_defs])
 
     @property
     def has_configurable_outputs(self):
-        return any([out.dagster_type.output_materialization_config for out in self._output_defs])
+        return any([out.dagster_type.materializer for out in self._output_defs])
 
     @abstractproperty
     def has_config_entry(self):
@@ -160,10 +161,15 @@ class ISolidDefinition(six.with_metaclass(ABCMeta)):
 
         return CallableSolidNode(self, tags=validate_tags(tags))
 
+    def with_hooks(self, hook_defs):
+        from .composition import CallableSolidNode
 
-class SolidDefinition(ISolidDefinition):
+        return CallableSolidNode(self, hook_defs=hook_defs)
+
+
+class SolidDefinition(ISolidDefinition, IConfigMappable):
     '''
-    The definition of a Solid that peforms a user defined computation.
+    The definition of a Solid that performs a user-defined computation.
 
     For more details on what a solid is, refer to the
     `Solid Guide <../../learn/guides/solid/solid>`_ .
@@ -184,31 +190,8 @@ class SolidDefinition(ISolidDefinition):
             of the solid's ``output_defs``, and additionally may yield other types of Dagster
             events, including :py:class:`Materialization` and :py:class:`ExpectationResult`.
         output_defs (List[OutputDefinition]): Outputs of the solid.
-        config (Optional[Any]): The schema for the config. Configuration data available in
-            `init_context.solid_config`.
-
-            This value can be any of:
-
-            1. A Python primitive type that resolves to a Dagster config type
-               (:py:class:`~python:int`, :py:class:`~python:float`, :py:class:`~python:bool`,
-               :py:class:`~python:str`, or :py:class:`~python:list`).
-
-            2. A Dagster config type: :py:data:`~dagster.Int`, :py:data:`~dagster.Float`,
-               :py:data:`~dagster.Bool`, :py:data:`~dagster.String`,
-               :py:data:`~dagster.StringSource`, :py:data:`~dagster.Path`, :py:data:`~dagster.Any`,
-               :py:class:`~dagster.Array`, :py:data:`~dagster.Noneable`, :py:data:`~dagster.Enum`,
-               :py:class:`~dagster.Selector`, :py:class:`~dagster.Shape`, or
-               :py:class:`~dagster.Permissive`.
-
-            3. A bare python dictionary, which will be automatically wrapped in
-               :py:class:`~dagster.Shape`. Values of the dictionary are resolved recursively
-               according to the same rules.
-
-            4. A bare python list of length one which itself is config type.
-               Becomes :py:class:`Array` with list element as an argument.
-
-            5. An instance of :py:class:`~dagster.Field`.
-
+        config_schema (Optional[ConfigSchema): The schema for the config. Configuration data
+            available in `init_context.solid_config`.
         description (Optional[str]): Human-readable description of the solid.
         tags (Optional[Dict[str, Any]]): Arbitrary metadata for the solid. Frameworks may
             expect and require certain metadata to be attached to a solid. Users should generally
@@ -218,6 +201,8 @@ class SolidDefinition(ISolidDefinition):
             solid.
         positional_inputs (Optional[List[str]]): The positional order of the input names if it
             differs from the order of the input definitions.
+        _configured_config_mapping_fn: This argument is for internal use only. Users should not
+            specify this field. To preconfigure a resource, use the :py:func:`configured` API.
 
 
     Examples:
@@ -240,25 +225,28 @@ class SolidDefinition(ISolidDefinition):
         input_defs,
         compute_fn,
         output_defs,
-        config=None,
+        config_schema=None,
         description=None,
         tags=None,
         required_resource_keys=None,
         positional_inputs=None,
-        metadata=None,
+        _configured_config_mapping_fn=None,
     ):
         self._compute_fn = check.callable_param(compute_fn, 'compute_fn')
-        self._config_field = check_user_facing_opt_config_param(config, 'config')
+        self._config_schema = check_user_facing_opt_config_param(config_schema, 'config_schema')
         self._required_resource_keys = frozenset(
             check.opt_set_param(required_resource_keys, 'required_resource_keys', of_type=str)
         )
+        self.__configured_config_mapping_fn = check.opt_callable_param(
+            _configured_config_mapping_fn, 'config_mapping_fn'
+        )
 
         super(SolidDefinition, self).__init__(
             name=name,
             input_defs=check.list_param(input_defs, 'input_defs', InputDefinition),
             output_defs=check.list_param(output_defs, 'output_defs', OutputDefinition),
             description=description,
-            tags=canonicalize_backcompat_args(tags, 'tags', metadata, 'metadata'),
+            tags=check.opt_dict_param(tags, 'tags', key_type=str),
             positional_inputs=positional_inputs,
         )
 
@@ -268,7 +256,12 @@ class SolidDefinition(ISolidDefinition):
 
     @property
     def config_field(self):
-        return self._config_field
+        rename_warning('config_schema', 'config_field', '0.9.0')
+        return self._config_schema
+
+    @property
+    def config_schema(self):
+        return self._config_schema
 
     @property
     def required_resource_keys(self):
@@ -276,14 +269,7 @@ class SolidDefinition(ISolidDefinition):
 
     @property
     def has_config_entry(self):
-        return self._config_field or self.has_configurable_inputs or self.has_configurable_outputs
-
-    def all_runtime_types(self):
-        rename_warning(
-            new_name='all_dagster_types', old_name='all_runtime_types', breaking_version='0.8.0'
-        )
-        for tt in self.all_input_output_types():
-            yield tt
+        return self._config_schema or self.has_configurable_inputs or self.has_configurable_outputs
 
     def all_dagster_types(self):
         for tt in self.all_input_output_types():
@@ -301,6 +287,28 @@ class SolidDefinition(ISolidDefinition):
     def default_value_for_input(self, input_name):
         return self.input_def_named(input_name).default_value
 
+    @property
+    def _configured_config_mapping_fn(self):
+        return self.__configured_config_mapping_fn
+
+    def configured(self, config_or_config_fn, config_schema=None, **kwargs):
+        wrapped_config_mapping_fn = self._get_wrapped_config_mapping_fn(
+            config_or_config_fn, config_schema
+        )
+
+        return SolidDefinition(
+            name=kwargs.get('name', self.name),
+            input_defs=self.input_defs,
+            compute_fn=self.compute_fn,
+            output_defs=self.output_defs,
+            config_schema=config_schema,
+            description=kwargs.get('description', self.description),
+            tags=self.tags,
+            required_resource_keys=self.required_resource_keys,
+            positional_inputs=self.positional_inputs,
+            _configured_config_mapping_fn=wrapped_config_mapping_fn,
+        )
+
 
 class CompositeSolidDefinition(ISolidDefinition, IContainSolids):
     '''The core unit of composition and abstraction, composite solids allow you to
@@ -551,17 +559,6 @@ class CompositeSolidDefinition(ISolidDefinition, IContainSolids):
 
         return mapped_solid.definition.default_value_for_input(mapping.input_name)
 
-    def all_runtime_types(self):
-        rename_warning(
-            new_name='all_dagster_types', old_name='all_runtime_types', breaking_version='0.8.0'
-        )
-        for tt in self.all_input_output_types():
-            yield tt
-
-        for solid_def in self._solid_defs:
-            for ttype in solid_def.all_dagster_types():
-                yield ttype
-
     def all_dagster_types(self):
         for tt in self.all_input_output_types():
             yield tt
