@@ -118,7 +118,7 @@ def create_run_for_test(
     instance,
     pipeline_name=None,
     run_id=None,
-    environment_dict=None,
+    run_config=None,
     mode=None,
     solids_to_execute=None,
     step_keys_to_execute=None,
@@ -133,7 +133,7 @@ def create_run_for_test(
     return instance.create_run(
         pipeline_name,
         run_id,
-        environment_dict,
+        run_config,
         mode,
         solids_to_execute,
         step_keys_to_execute,
@@ -151,7 +151,7 @@ def register_managed_run_for_test(
     instance,
     pipeline_name=None,
     run_id=None,
-    environment_dict=None,
+    run_config=None,
     mode=None,
     solids_to_execute=None,
     step_keys_to_execute=None,
@@ -165,7 +165,7 @@ def register_managed_run_for_test(
     return instance.register_managed_run(
         pipeline_name,
         run_id,
-        environment_dict,
+        run_config,
         mode,
         solids_to_execute,
         step_keys_to_execute,
